#!/usr/bin/python3
"""
CommentConvert - Converts comments to html file
"""

import sys; sys.dont_write_bytecode = True
import argparse
import json
import os
import shutil

from commentModels import ScrappedComment
import convertor
from convertor import HtmlConvertor

VERSION = "0.4.1"


def U_printConversionHelp():
    """Print conversion help."""
    print("When converting JSON to HTML there are several ways of doing so")
    print("(0) - Basic - All comments in JSON are extracted to single HTML"
          + " - has performance issues with large ammount of comments")
    print("(1) - PagedDynamic - Just like (1) but pages are loaded using iframes so video will stay loaded"
          + "- may have compatibility issues with older browsers")


# Will be used later to a decide whether we want to write universalcomment json or just html json
# Currently unused since universal comment is not implemented yet
def U_isOutputHtml(out):
    """Automatically determines whether we want implicit --output-html or  --output-json."""
    if not out:
        print("Some output file required!")
        sys.exit(1)
    if out.endswith("html"):
        return True
    elif out.endswith("json"):
        return False
    else:
        print("Cannot determine file using extension! Either use (--output-html or --output-json)"
              + " or the correct .extension")
        sys.exit(1)


def __parseArgs(args):
    # The unimplemented options are mostly commented out to not waste space in help message
    parser = argparse.ArgumentParser(description="Converts comments json file into universal comment "
                                     + "and/or html\n Ver: " + VERSION)
    parser.add_argument("--input", "-i", type=argparse.FileType("r"), required=True,
                        help="Input JSON with comments. Type will be resolved automatically but please"
                        + " specify type with '-a' or '-s' if possible")
    parser.add_argument("--api", "-a", action="store_true", help="Given input file is NOT scrapped by"
                        + " youtube_comment_downloader but from youtube API")
    parser.add_argument("--output", "-o", type=str, help="Output file, whether its universal comment JSON"
                        + " or HTML is decided by extension. Default is [input].html")
    parser.add_argument("--videos", "-v", type=str, nargs="+", default=list(),
                        help="Path to the video(s) that should be linked the resulting HTML file, first"
                        + " having the highest load priority")
    parser.add_argument("--scripts", "-s", type=str, nargs="+", default=list(),
                        help="Path to the JS script(s) that should be linked the resulting HTML file"
                        + " - replaces defaults, is copied to working dir if it's not in there already")
    parser.add_argument("--styles", "-c", type=str, nargs="+", default=list(),
                        help="Path to the CSS style(s) that should be linked the resulting HTML file"
                        + " - replaces defaults, is copied to working dir if it's not in there already")
    parser.add_argument("--no-link-default", "-nld", action="store_true", dest="noLinkDefault",
                        help="Wont link default JS and CSS thus producing clean html", default=False)
    parser.add_argument("--paging-comments", "-p", type=int, default=0, dest="commentsPerPage",
                        help="Makes new html page (roughly) every given number of comments")
    parser.add_argument("--download-avatars", "-d", type=str, nargs="?", const="./avatars/", dest="avatarDir",
                        help="Path to directory where avatars should be downloaded. If ommited,"
                        + " no avatars are downloaded")
#    parser.add_argument("--html-conversion-mode", "-hcm", type=int,
#                        help="Number 0-3 Specifies how should the resulting HTML should look (default 0),"
#                              + " for more info check --conversion-help or README.md")
#    parser.add_argument("--conversion-help", action="store_true",
#                        help="Prints additional information for conversion between JSON and html")

    return parser.parse_args()


def smartCommentParse(reader):
    """Parse comments in more advanced way to pair replies to their parent."""
    print("Parsing JSON...")
    commentList = list()
    line = reader.readline()
    lastRoot = 0
    lineCount = 0  # Maybe we could display number of coments in resulting html?

    while not line == "":
        sys.stdout.write("Parsing line: " + str(lineCount) + "\r")
        comment = ScrappedComment(json.loads(line))

        if comment.isReply:  # we have to find parent to this comment
            if comment.getCid() == commentList[lastRoot].getCid():  # Replies are often comming right after
                commentList[lastRoot].addReply(comment)             # each other so we can check if we haven't
                continue                                            # found it already and save iterations

            for i in range(0, len(commentList)):  # for parent to this comment
                if commentList[i].getCid() == comment.getParent():
                    commentList[i].addReply(comment)  # I Wonder what will happen if we wont find it
                    lastRoot = i
                    break
            # This was originally here to check if we went entire loop without figuring out where the parent is but it's actually not that easy to catch it
            # So if problems would arise here is error template i guess. But at this point it prints everytime we pass reply
            # print("WARNING:<smartCommentParse>: This should not happen! We have reply without parent somewhere in commentList.")
            # print("index is somewhere around: " + str(len(commentList) - 1) + " (Actual line is: " + str(lineCount) + " cid: " + comment.getRawCid() + ")")
        else:
            commentList.append(comment)
        line = reader.readline()
        lineCount += 1
    reader.close()
    return commentList


def copyAssets(out, args):  # Only control flow goto-hack function, need refactoring, I know it's horrible
    # We have to copy these files to the script working dir from the script location dir
    scriptHome = os.path.realpath(__file__)  # Dir where the script is located NOT where it's run from
    scriptHome = scriptHome[:scriptHome.rfind("/") + 1]
    assetDir = "./"
    if "/" in out:
        ced = out[:out.rfind("/") + 1]   # Content extraction dir
    else:
        ced = "./"
    if (not os.path.exists(args.styles[0])) or (not os.path.exists(args.scripts[0])):
        # pokud ano tak je prostě zkopčíme z ., pokud ne, kopírujeme z script home
        if os.path.exists(scriptHome + args.styles[0]) or os.path.exists(scriptHome + args.scripts[0]):
            assetDir = scriptHome
        else:
            print("Error: default 'script.js' or 'style.css' is missing from this scripts location ("
                  + scriptHome + ")")
            print("No default assets were copied")
            return
        if ("../" in ced):  # Happens when reaching for something to upper directories
            ced = "./"      # this is retarded, what if there are two ".."
        try:
            shutil.copy(assetDir + args.styles[0], ced)
            shutil.copy(assetDir + args.scripts[0], ced)
        except shutil.SameFileError:
            pass  # Files are already copied, nothing needs to be done


def main(argv=None):
    args = __parseArgs(argv)

    if args.api:
        print("API comments are not supported yet")
        sys.exit(1)

    comlist = smartCommentParse(args.input)  # We are sure it's not api file now
    conv = HtmlConvertor(comlist)

    # BUG: This will cut out last letter if the input has no "." in name
    out = args.output if args.output else args.input.name[:args.input.name.rfind(".")] + ".html"

    if not args.noLinkDefault:
        # We can append as argparser should always initialize the lists for us
        # Prepending as we want additional files to have higher priority for better customization
        args.styles.insert(0, "style.css")
        args.scripts.insert(0, "script.js")
        copyAssets(out, args)

    conv.videos = args.videos
    conv.scripts = args.scripts
    conv.styles = args.styles
    if args.avatarDir is None:
        conv.Avatar_Action = convertor.AVATAR_NONE
    else:
        conv.avatarDir = args.avatarDir if args.avatarDir.endswith("/") else args.avatarDir + "/"
        conv.Avatar_Action = convertor.AVATAR_DOWNLOAD

    conv.Max_Comments_Per_Page = args.commentsPerPage

    conv.export(out)
    print("All jobs done, exiting...")


if __name__ == "__main__":
    main(sys.argv[1:])
